from zope.interface import Interface
from zope.authentication import interfaces as authentication


class IAuthentication(authentication.IAuthentication,
                      authentication.ILogout,
                      authentication.ILogoutSupported):
    pass


class ICredentialsPlugin(Interface):
    """Handles credentials extraction and challenges per request.
    """

    def extractCredentials(request):
        """Ties to extract credentials from a request.

        A return value of None indicates that no credentials could be found.
        Any other return value is treated as valid credentials.
        """

    def challenge(request):
        """Possibly issues a challenge.

        This is typically done in a protocol-specific way.

        If a challenge was issued, return True, otherwise return False.
        """

    def logout(request):
        """Possibly logout.

        If a logout was performed, return True, otherwise return False.
        """


class IAuthenticatorPlugin(Interface):
    """ Authenticates a principal using credentials.
    An authenticator may also be responsible for providing information
    about and creating principals.
    """

    def authenticateCredentials(credentials):
        """Authenticates credentials.

        If the credentials can be authenticated, return an object that provides
        IPrincipalInfo. If the plugin cannot authenticate the credentials,
        returns None.
        """

    def principalInfo(id):
        """Returns an IPrincipalInfo object for the specified principal id.

        If the plugin cannot find information for the id, returns None.
        """


class IAuthenticatedPrincipalFactory(Interface):

    def __init__(self, principal, request):
        """ MultiAdapter for IPrincipal and IRequest
        """

    def __call__(self, auth):
        """ auth is a instance thats provide IAuthentication
        """



from grokcore import component

from zope.interface import implementer
from zope.component import queryUtility
from zope.component.hooks import getSite
from zope.component import getMultiAdapter

from zope.authentication.interfaces import PrincipalLookupError

from bst.pygasus.security import interfaces
from bst.pygasus.security.interaction import ANONYMOUSE
from bst.pygasus.core.interfaces import IApplicationContext

from zope.authentication.interfaces import IAuthentication


@implementer(interfaces.IAuthentication)
class PluggableUtility(component.GlobalUtility):
    """ Default plugable authentication utility
        with similar functionality as zope framework.
    """

    def _credentials_pluggins(self):
        application = getSite()
        if application is None:
            return None
        for name in application.credentials_pluggins:
            pluggin = queryUtility(interfaces.ICredentialsPlugin, name=name)
            if pluggin is not None:
                yield pluggin

    def _authentication_pluggins(self):
        application = getSite()
        if application is None:
            return None
        for name in application.authentication_pluggins:
            pluggin = queryUtility(interfaces.IAuthenticatorPlugin, name=name)
            if pluggin is not None:
                yield pluggin

    def authenticate(self, request):
        for cred in self._credentials_pluggins():
            for auth in self._authentication_pluggins():
                credential_dict = cred.extractCredentials(request)
                if credential_dict is None:
                    continue
                principal = auth.authenticateCredentials(credential_dict)
                if principal is not None:
                    return getMultiAdapter((principal, request),
                                           interfaces.IAuthenticatedPrincipalFactory)(self)
        return None

    def unauthenticatedPrincipal(self):
        return ANONYMOUSE

    def unauthorized(self, id, request):
        raise NotimplementedError('just not implemented at the moment')

    def getPrincipal(self, id):
        for auth in self._authentication_pluggins():
            principal = auth.principalInfo(id)
            if principal is None:
                continue
            return principal
        if self.unauthenticatedPrincipal() is not None:
            return self.unauthenticatedPrincipal()
        raise PrincipalLookupError(id)

    def logout(self, request):
        for cred in self._credentials_pluggins():
            if cred.logout(request):
                return True
        return False



from bst.pygasus.core import ext
from js.extjs.theme import themes

from zope import schema
from zope.interface import Interface
from zope.component import getUtility

from fanstatic import Library
from fanstatic import Resource

from bst.pygasus.security import _
from bst.pygasus.security.plugins import FORM_LOGIN
from bst.pygasus.security.plugins import FORM_PASSWORD
from bst.pygasus.security.interfaces import IAuthentication
from bst.pygasus.core.interfaces import DEFAULT_EXTJS_APPLICATION


library = Library('securtiylogin', 'applogin')
style = Resource(library, 'resources/style.css')


class LoginPageContext(ext.ApplicationContext):
    """ Base class to easily create a login page in your
        application. All you need is inherited in your project.
    """
    ext.baseclass()
    ext.name('login')

    title = 'Login Page'
    application = 'extjs.security.LoginPageApplication'
    namespace = 'extjs.security'
    resources = Resource(library, 'application.js',
                         depends=[ext.extjs_resources_skinless,
                                  themes['neptune'],
                                  style])

    credentials_pluggins = ('request_credentials',)
    authentication_pluggins = ()


class AppClassPathMapping(ext.ClassPathMapping):
    namespace = 'extjs.security'
    path = 'fanstatic/securtiylogin'


@ext.scaffolding('Credentials')
class ICredentials(Interface):

    login = schema.TextLine(title=_('tr_lbl_username', default='Username'),
                            required=True)

    password = schema.Password(title=_('tr_lbl_password', default='Password'),
                               required=True)

    success = schema.Bool(title='Success',
                          required=False)

    defaultredirect = schema.TextLine(title='default redirect',
                                      required=False)


class Credentials(ext.Model):
    ext.schema(ICredentials)
    login = ''
    password = ''
    success = False
    defaultredirect = DEFAULT_EXTJS_APPLICATION


class CredentialsHandler(ext.AbstractModelHandler):
    """ This handler will just push the credential to request. After that a
        credential-plugin will create the information for the auth-plugin.
    """
    ext.adapts(Credentials, ext.IRequest)

    def get(self, model, batch):
        """ just return a empty list
        """
        return [model], 1

    def create(self, model):
        raise NotImplementedError('not possible...')

    def delete(self, model):
        raise NotImplementedError('not possible...')

    def update(self, model, batch):
        self.request.GET.add(FORM_LOGIN, model.login)
        self.request.GET.add(FORM_PASSWORD, model.password)

        principal = getUtility(IAuthentication).authenticate(self.request)
        model.success = principal is not None
        return [model], 1



from grokcore import component
from zope.interface import implementer

from bst.pygasus.session.interfaces import ISession
from bst.pygasus.security.principal import Principal
from bst.pygasus.security.interfaces import ICredentialsPlugin
from bst.pygasus.security.interfaces import IAuthenticatorPlugin


REMOTEUSER = 'remoteuser'
FORM_LOGIN = 'loginform.login'
FORM_PASSWORD = 'loginform.password'
SESSION_CREDENTIALS = 'bst.pygasus.security.session.credentials'


@implementer(ICredentialsPlugin)
class RemoteCredentialsPlugin(component.GlobalUtility):
    """ Fetch remote user from environment variable. This variable must
        be set via rewrite proxy e.g. apache. Normally this plugin
        works with RemoteAuthenticatorPlugin.
    """
    component.name('remoteuser')

    def extractCredentials(self, request):
        """ fetch remote user in the nvironment variable
            and create credentials with it.
        """
        remote_user = request.environ.get('REMOTE_USER', None)
        if remote_user is None:
            return None
        return {REMOTEUSER: remote_user}

    def challenge(self, request):
        return False

    def logout(self, request):
        return False


@implementer(ICredentialsPlugin)
class RequestCredentialsPlugin(component.GlobalUtility):
    """ Fetch credentials from HTTP headers.
    """
    component.name('request_credentials')

    def extractCredentials(self, request):
        """ fetch remote user in the http headers
            and create credentials with it.
        """
        session = ISession(request)
        cred = dict(login=request.params.get(FORM_LOGIN, None),
                    password=request.params.get(FORM_PASSWORD, None))
        if None in cred.values():
            if SESSION_CREDENTIALS in session:
                return session[SESSION_CREDENTIALS]
            else:
                return None
        session[SESSION_CREDENTIALS] = cred
        return cred

    def challenge(self, request):
        return False

    def logout(self, request):
        session = ISession(request)
        re = SESSION_CREDENTIALS in session
        del session[SESSION_CREDENTIALS]
        return re


@implementer(IAuthenticatorPlugin)
class RemoteAuthenticatorPlugin(component.GlobalUtility):

    component.name('remoteuser')

    def authenticateCredentials(self, credentials):
        if REMOTEUSER not in credentials:
            return None
        return self.principalInfo(credentials[REMOTEUSER])

    def principalInfo(self, id):
        return Principal(id)



from grokcore import component

from zope.security import management
from zope.component import getUtility
from zope.interface import implementer
from zope.security.interfaces import IParticipation
from zope.securitypolicy.zopepolicy import ZopeSecurityPolicy

from bst.pygasus.security import _
from bst.pygasus.security.principal import Principal
from bst.pygasus.security.interfaces import IAuthentication
from bst.pygasus.core.interfaces import IApplicationContext
from bst.pygasus.wsgi.events import IApplicationStartupEvent
from bst.pygasus.wsgi.interfaces import IApplicationSettings
from bst.pygasus.wsgi.events import IPreRequestProcessingEvent
from bst.pygasus.wsgi.events import IPostRequestProcessingEvent


ANONYMOUSE = Principal('bst.pygasus.anonymouse_user',
                       _('Anonymouse'),
                       _('Anonymouse user that is not logged in'))

AUTHENTICATED = Principal('bst.pygasus.authenticated_user',
                          _('Authenticated'),
                          _('Authenticated user that we know'))


@implementer(IParticipation)
class Participation(object):

    def __init__(self, principal):
        self.principal = principal
        # Note the interaction will be set by SecurityPolicy
        self.interaction = None


@component.subscribe(IApplicationSettings, IApplicationStartupEvent)
def set_policy_security(settings, event):
    management.setSecurityPolicy(ZopeSecurityPolicy)


# not finish at all, role and rights are not supported at the moment
"""
@component.subscribe(IApplicationContext, IPreRequestProcessingEvent)
def new_interaction(context, event):
    participations = list()
    principal = getUtility(IAuthentication).authenticate(event.request)
    if principal is not None:
        participations.append(Participation(principal))
        participations.append(Participation(AUTHENTICATED))
    else:
        participations.append(Participation(ANONYMOUSE))

    management.newInteraction(*participations)


@component.subscribe(IApplicationContext, IPostRequestProcessingEvent)
def end_interaction(context, event):
    management.endInteraction()
"""



from grokcore import component
from zope.interface import implementer
from zope.security.interfaces import IPrincipal

from bst.pygasus.wsgi.interfaces import IRequest
from bst.pygasus.security.interfaces import IAuthenticatedPrincipalFactory


@implementer(IPrincipal)
class Principal(object):

    def __init__(self, id, title=None, description=None):
        self.id = id
        self.title = title
        self.description = description


@implementer(IAuthenticatedPrincipalFactory)
class SimpleAuthenticatedPrincipalFactory(component.MultiAdapter):
    """ Default principal factory thats do nothing
        else as return the same principal. If you need to
        transform your principal so override this adapter.
    """

    component.adapts(IPrincipal, IRequest)

    def __init__(self, principal, request):
        self.principal = principal
        self.request = request

    def __call__(self, auth):
        return self.principal



from setuptools import setup, find_packages
import os

version = '1.0'

long_description = (
    open('README.rst').read()
    + '\n' +
    open('CHANGES.rst').read()
    + '\n')

setup(name='bst.pygasus.security',
      version=version,
      description="provide authentication for bst.pygasus framework",
      long_description=long_description,
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          'Programming Language :: Python :: 3',
          'Natural Language :: English',
          'License :: OSI Approved :: Zope Public License',
          'Operating System :: OS Independent',
          'Development Status :: 4 - Beta'
      ],
      keywords='pygasus web framework extjs',
      author='codeix',
      author_email='samuel.riolo@biel-bienne.ch',
      url='https://github.com/bielbienne/bst.pygasus.security',
      license='ZPL 2.1',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['bst', 'bst.pygasus'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'zope.authentication',
          # remove this pin after a final version!
          'zope.principalregistry == 4.0.0a2',
          'zope.securitypolicy == 4.0.0a1',
          'bst.pygasus.session'
          # -*- Extra requirements: -*-
      ],
      entry_points={
          'fanstatic.libraries': ['loginsecurity = bst.pygasus.security.extjs:library'],
      },
      )
